ShotDs.Data.Substitution (shot_ds v1.0.0)

Copy Markdown View Source

Represents a substitution.

Summary

Types

t()

The type of a substitution.

Functions

Creates a new substitution for the given free variable and the id for its replacement term.

Types

t()

@type t() :: %ShotDs.Data.Substitution{
  fvar: ShotDs.Data.Declaration.free_var_t(),
  term_id: ShotDs.Data.Term.term_id()
}

The type of a substitution.

A substitution is made up by the free variable to substitute and the id for its replacement term.

Functions

new(fvar, term_id)

Creates a new substitution for the given free variable and the id for its replacement term.