ShotDs.Hol.Patterns (shot_ds v1.0.0)

Copy Markdown View Source

Contains various macros for pattern matching on HOL terms.

Summary

Functions

Matches a term representing the conjunction of p_id and q_id.

Matches a term representing the disjunction of p_id and q_id.

Matches a term representing the equality of p_id and q_id.

Matches a term representing the equivalence of p_id and q_id.

Matches a term representing the universal quantification of the predicate body_id.

Matches a term representing falsity.

Matches a term representing the implication of p_id and q_id, i.e. p implies q.

Matches a term representing the negation of term_id.

Matches a term representing truth.

Matches a term representing the equality of p_id and q_id with respect to their type.

Matches a term representing the universal quantification of the predicate body_id with respect to the element type type.

Matches a term representing the universal quantification of the predicate body_id with respect to the element type type.

Matches a term representing the universal quantification of the predicate body_id.

Functions

conjunction(p_id, q_id)

(macro)

Matches a term representing the conjunction of p_id and q_id.

disjunction(p_id, q_id)

(macro)

Matches a term representing the disjunction of p_id and q_id.

equality(a_id, b_id)

(macro)

Matches a term representing the equality of p_id and q_id.

equivalence(p_id, q_id)

(macro)

Matches a term representing the equivalence of p_id and q_id.

existential_quantification(body_id)

(macro)

Matches a term representing the universal quantification of the predicate body_id.

falsity()

(macro)

Matches a term representing falsity.

implication(p_id, q_id)

(macro)

Matches a term representing the implication of p_id and q_id, i.e. p implies q.

negated(term_id)

(macro)

Matches a term representing the negation of term_id.

truth()

(macro)

Matches a term representing truth.

typed_equality(a_id, b_id, type)

(macro)

Matches a term representing the equality of p_id and q_id with respect to their type.

typed_existential_quantification(body_id, type)

(macro)

Matches a term representing the universal quantification of the predicate body_id with respect to the element type type.

typed_universal_quantification(body_id, type)

(macro)

Matches a term representing the universal quantification of the predicate body_id with respect to the element type type.

universal_quantification(body_id)

(macro)

Matches a term representing the universal quantification of the predicate body_id.