Represents a substitution.
Summary
Functions
Creates a new substitution for the given free variable and the id for its replacement term.
Types
@type t() :: %ShotDs.Data.Substitution{ fvar: ShotDs.Data.Declaration.free_var_t(), term_id: ShotDs.Data.Term.term_id() }
The type of a substitution.
A substitution is made up by the free variable to substitute and the id for its replacement term.
Functions
@spec new(ShotDs.Data.Declaration.free_var_t(), ShotDs.Data.Term.term_id()) :: t()
Creates a new substitution for the given free variable and the id for its replacement term.